"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteBookmark = exports.updateBookmark = exports.addBookmark = exports.getBookmarks = exports.getBookmark = void 0;
const bookmark_record_1 = require("../records/bookmark.record");
const errors_1 = require("../utils/errors");
// @desc Get single bookmark
// @route GET /bookmarks/:id
// @access Private
const getBookmark = async (req, res) => {
    const { user } = req.session;
    if (!user)
        throw new errors_1.AuthError();
    const bookmark = await bookmark_record_1.BookmarkRecord.getOne(req.params.id);
    if (!bookmark)
        throw new errors_1.ValidationError('Bookmark not found');
    if (user.id !== bookmark.user)
        throw new errors_1.AuthError('User unauthorized to access requested bookmark');
    res.status(200).json(bookmark);
};
exports.getBookmark = getBookmark;
// @desc Get all bookmarks
// @route GET /bookmarks
// @access Private
const getBookmarks = async (req, res) => {
    const { name, favorites } = req.query;
    const { user } = req.session;
    if (!user)
        throw new errors_1.AuthError();
    const bookmarks = await bookmark_record_1.BookmarkRecord.getAllByUser(user.id, name && name.toString(), favorites === '' || !!favorites);
    res.status(200).json(bookmarks);
};
exports.getBookmarks = getBookmarks;
// @desc Add bookmark
// @route POST /bookmarks
// @access Private
const addBookmark = async (req, res) => {
    const { user } = req.session;
    if (!user)
        throw new errors_1.AuthError();
    const bookmark = new bookmark_record_1.BookmarkRecord({ ...req.body, user: user.id });
    await bookmark.add();
    res.status(201).json(bookmark);
};
exports.addBookmark = addBookmark;
// @desc Update bookmark
// @route PUT /bookmarks/:id
// @access Private
const updateBookmark = async (req, res) => {
    const { name, url, favorite } = req.body;
    const { user } = req.session;
    if (!user)
        throw new errors_1.AuthError();
    let bookmark = await bookmark_record_1.BookmarkRecord.getOne(req.params.id);
    if (!bookmark)
        throw new errors_1.ValidationError('Bookmark not found');
    if (user.id !== bookmark.user)
        throw new errors_1.AuthError('User unauthorized to access requested bookmark');
    if (bookmark.name !== undefined)
        bookmark.name = name;
    if (bookmark.url !== undefined)
        bookmark.url = url;
    if (bookmark.favorite !== undefined)
        bookmark.favorite = favorite;
    bookmark = await bookmark.update();
    res.status(200).json(bookmark);
};
exports.updateBookmark = updateBookmark;
// @desc Delete bookmark
// @route DELETE /bookmarks/:id
// @access Private
const deleteBookmark = async (req, res) => {
    const { user } = req.session;
    if (!user)
        throw new errors_1.AuthError();
    const bookmark = await bookmark_record_1.BookmarkRecord.getOne(req.params.id);
    if (!bookmark)
        throw new errors_1.ValidationError('Bookmark not found');
    if (user.id !== bookmark.user)
        throw new errors_1.AuthError('User unauthorized to access requested bookmark');
    await bookmark.delete();
    res.status(200).end();
};
exports.deleteBookmark = deleteBookmark;
//# sourceMappingURL=bookmarks.js.map