"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BookmarkRecord = void 0;
const uuid_1 = require("uuid");
const db_1 = require("../utils/db");
const errors_1 = require("../utils/errors");
class BookmarkRecord {
    id;
    name;
    url;
    favorite = false;
    user;
    constructor({ id, name, url, favorite, user }) {
        if (!name) {
            throw new errors_1.ValidationError('Bookmark name is required');
        }
        if (name.length > 200) {
            throw new errors_1.ValidationError('Bookmark name cannot be more than 200 characters');
        }
        if (!url) {
            throw new errors_1.ValidationError('Bookmark url is required');
        }
        if (url.length > 500) {
            throw new errors_1.ValidationError('Bookmark url cannot be more that 500 characters');
        }
        if (!user) {
            throw new errors_1.ValidationError('Bookmark must be assigned to user');
        }
        this.id = id ?? (0, uuid_1.v4)();
        this.name = name;
        this.url = url;
        this.favorite = !!favorite;
        this.user = user;
    }
    async add() {
        await db_1.pool.execute('INSERT INTO `bookmarks`(`id`,`name`,`url`,`favorite`,`user`) VALUES(:id, :name, :url, :favorite, :user)', this);
        return this;
    }
    async update() {
        await db_1.pool.execute('UPDATE `bookmarks` SET `name` = :name, `url` = :url, `favorite` = :favorite WHERE `id` = :id', this);
        return this;
    }
    async delete() {
        await db_1.pool.execute('DELETE FROM `bookmarks` WHERE `id` = :id', this);
    }
    static async getOne(id) {
        const [results] = (await db_1.pool.execute('SELECT * from `bookmarks` WHERE `id` = :id', {
            id,
        }));
        if (!results[0])
            return null;
        return new BookmarkRecord(results[0]);
    }
    static async getAll(name, favorites) {
        if (name && favorites) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `name` LIKE :name && `favorite` = 1 ORDER BY `createdAt` DESC', {
                name: `%${name}%`,
            }));
            return results.map((result) => new BookmarkRecord(result));
        }
        if (name) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `name` LIKE :name ORDER BY `createdAt` DESC', { name: `%${name}%` }));
            return results.map((result) => new BookmarkRecord(result));
        }
        if (favorites) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `favorite` = 1 ORDER BY `createdAt` DESC'));
            return results.map((result) => new BookmarkRecord(result));
        }
        const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` ORDER BY `createdAt` DESC'));
        return results.map((result) => new BookmarkRecord(result));
    }
    static async getAllByUser(user, name, favorites) {
        if (name && favorites) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `user` = :user && `name` LIKE :name && `favorite` = 1 ORDER BY `createdAt` DESC', {
                user,
                name: `%${name}%`,
            }));
            return results.map((result) => new BookmarkRecord(result));
        }
        if (name) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `user` = :user && `name` LIKE :name ORDER BY `createdAt` DESC', {
                user,
                name: `%${name}%`,
            }));
            return results.map((result) => new BookmarkRecord(result));
        }
        if (favorites) {
            const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `user` = :user && `favorite` = 1 ORDER BY `createdAt` DESC', {
                user,
            }));
            return results.map((result) => new BookmarkRecord(result));
        }
        const [results] = (await db_1.pool.execute('SELECT `id`, `name`, `url`, `favorite`, `user` FROM `bookmarks` WHERE `user` = :user ORDER BY `createdAt` DESC', {
            user,
        }));
        return results.map((result) => new BookmarkRecord(result));
    }
}
exports.BookmarkRecord = BookmarkRecord;
//# sourceMappingURL=bookmark.record.js.map